<?php

namespace Drupal\convert_entity_media_embed\Plugin\QueueWorker;

use Drupal\convert_entity_media_embed\FieldTypes;
use Drupal\Core\Entity\EntityFieldManagerInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Queue\QueueFactory;
use Drupal\Core\Queue\QueueWorkerBase;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Finds entities with values for a field and builds a queue for them.
 *
 * @QueueWorker(
 *   id = "convert_entity_media_embed_entities",
 *   title = @Translation("Find entities for fields."),
 *   cron = {"time" = 60}
 * )
 */
class EntityQueueWorker extends QueueWorkerBase implements ContainerFactoryPluginInterface {

  /**
   * The queue factory.
   *
   * @var \Drupal\Core\Queue\QueueFactory
   */
  protected $queueFactory;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The entity field manager.
   *
   * @var \Drupal\Core\Entity\EntityFieldManagerInterface
   */
  protected $fieldManager;

  /**
   * {@inheritdoc}
   */
  public function processItem($data) {
    $entityTypeId = $data['entity_type'];
    $entityId = $data['id'];
    $replacementPerformed = FALSE;

    $entityStorage = $this->entityTypeManager->getStorage($entityTypeId);

    /** @var \Drupal\Core\Entity\ContentEntityInterface $entity */
    $entity = $entityStorage->load($entityId);

    if (empty($entity)) {
      return;
    }

    $fieldDefinitions = $entity->getFieldDefinitions();

    foreach ($entity->getTranslationLanguages() as $langcode => $language) {
      $translation = $entity->getTranslation($langcode);

      foreach ($fieldDefinitions as $fieldDefinition) {
        if (!in_array($fieldDefinition->getType(), FieldTypes::TYPES)) {
          continue;
        }

        $fieldItemList = $translation->get($fieldDefinition->getName());
        if ($fieldItemList->isEmpty()) {
          continue;
        }

        for ($i = 0; $i < $fieldItemList->count(); $i++) {
          $item = $fieldItemList->get($i);
          $value = $item->getValue();
          $originalValue = $value['value'];
          $value['value'] = preg_replace('/<drupal-entity(.*)data-entity-type="media"(.*)><\/drupal-entity>/', '<drupal-media${1}data-entity-type="media"${2}></drupal-media>', $originalValue);

          if ($value['value'] != $originalValue) {
            $item->setValue($value);
            $fieldItemList->set($i, $item);
            $replacementPerformed = TRUE;
          }
        }
      }
    }

    if ($replacementPerformed) {
      $entity->save();
    }
  }

  /**
   * FieldQueueWorker constructor.
   *
   * @param array $configuration
   *   Plugin configuration.
   * @param string $plugin_id
   *   Plugin ID.
   * @param mixed $plugin_definition
   *   Plugin definition.
   * @param \Drupal\Core\Queue\QueueFactory $queueFactory
   *   The queue factory.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   The entity type manager.
   */
  public function __construct(
    array $configuration,
    string $plugin_id,
    $plugin_definition,
    QueueFactory $queueFactory,
    EntityTypeManagerInterface $entityTypeManager
  ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->queueFactory = $queueFactory;
    $this->entityTypeManager = $entityTypeManager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(
    ContainerInterface $container,
    array $configuration,
    $plugin_id,
    $plugin_definition
  ) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('queue'),
      $container->get('entity_type.manager')
    );
  }

}
